@@ -201,7 +201,7 @@ impl<'reg: 'rc, 'rc> RenderContext<'reg, 'rc> {
     fn get_local_var(&self, level: usize, name: &str) -> Option<&Json> {
         self.blocks
             .get(level)
-            .and_then(|blk| blk.get_local_var(&name))
+            .and_then(|blk| blk.get_local_var(name))
     }
 
     /// Test if given template name is current template.
@@ -277,7 +277,7 @@ impl<'reg, 'rc> fmt::Debug for RenderContextInner<'reg, 'rc> {
             .field("partial_block_stack", &self.partial_block_stack)
             .field("root_template", &self.root_template)
             .field("current_template", &self.current_template)
-            .field("disable_eacape", &self.disable_escape)
+            .field("disable_escape", &self.disable_escape)
             .finish()
     }
 }
@@ -1117,3 +1117,37 @@ fn test_zero_args_heler() {
         "Output name: first_name not resolved"
     );
 }
+
+#[test]
+fn test_identifiers_starting_with_numbers() {
+    let mut r = Registry::new();
+
+    assert!(r
+        .register_template_string("r1", "{{#if 0a}}true{{/if}}")
+        .is_ok());
+    let r1 = r.render("r1", &json!({"0a": true})).unwrap();
+    assert_eq!(r1, "true");
+
+    assert!(r.register_template_string("r2", "{{eq 1a 1}}").is_ok());
+    let r2 = r.render("r2", &json!({"1a": 2, "a": 1})).unwrap();
+    assert_eq!(r2, "false");
+
+    assert!(r
+        .register_template_string("r3", "0: {{0}} {{#if (eq 0 true)}}resolved from context{{/if}}\n1a: {{1a}} {{#if (eq 1a true)}}resolved from context{{/if}}\n2_2: {{2_2}} {{#if (eq 2_2 true)}}resolved from context{{/if}}") // YUP it is just eq that barfs! is if handled specially? maybe this test should go nearer to specific helpers that fail?
+        .is_ok());
+    let r3 = r
+        .render("r3", &json!({"0": true, "1a": true, "2_2": true}))
+        .unwrap();
+    assert_eq!(
+        r3,
+        "0: true \n1a: true resolved from context\n2_2: true resolved from context"
+    );
+
+    // these should all be errors:
+    assert!(r.register_template_string("r4", "{{eq 1}}").is_ok());
+    assert!(r.register_template_string("r5", "{{eq a1}}").is_ok());
+    assert!(r.register_template_string("r6", "{{eq 1a}}").is_ok());
+    assert!(r.render("r4", &()).is_err());
+    assert!(r.render("r5", &()).is_err());
+    assert!(r.render("r6", &()).is_err());
+}
