@@ -5,11 +5,11 @@ use std::str::FromStr;
 
 use pest::error::LineColLocation;
 use pest::iterators::Pair;
-use pest::{Parser, Position};
+use pest::{Parser, Position, Span};
 use serde_json::value::Value as Json;
 
 use crate::error::{TemplateError, TemplateErrorReason};
-use crate::grammar::{HandlebarsParser, Rule};
+use crate::grammar::{self, HandlebarsParser, Rule};
 use crate::json::path::{parse_json_path_from_iter, Path};
 
 use self::TemplateElement::*;
@@ -25,6 +25,21 @@ pub struct Template {
     pub mapping: Vec<TemplateMapping>,
 }
 
+#[derive(Default)]
+pub(crate) struct TemplateOptions {
+    pub(crate) prevent_indent: bool,
+    pub(crate) name: Option<String>,
+}
+
+impl TemplateOptions {
+    fn name(&self) -> String {
+        self.name
+            .as_ref()
+            .cloned()
+            .unwrap_or_else(|| "Unnamed".to_owned())
+    }
+}
+
 #[derive(PartialEq, Clone, Debug)]
 pub struct Subexpression {
     // we use box here avoid resursive struct definition
@@ -380,12 +395,40 @@ impl Template {
 
     fn remove_previous_whitespace(template_stack: &mut VecDeque<Template>) {
         let t = template_stack.front_mut().unwrap();
-        if let Some(el) = t.elements.pop() {
-            if let RawString(ref text) = el {
-                t.elements.push(RawString(text.trim_end().to_owned()));
-            } else {
-                t.elements.push(el);
+        if let Some(RawString(ref mut text)) = t.elements.last_mut() {
+            *text = text.trim_end().to_owned();
+        }
+    }
+
+    fn process_standalone_statement(
+        template_stack: &mut VecDeque<Template>,
+        source: &str,
+        current_span: &Span<'_>,
+        prevent_indent: bool,
+    ) -> bool {
+        let with_trailing_newline = grammar::starts_with_empty_line(&source[current_span.end()..]);
+
+        if with_trailing_newline {
+            let with_leading_newline =
+                grammar::ends_with_empty_line(&source[..current_span.start()]);
+
+            // prevent_indent: a special toggle for partial expression
+            // (>) that leading whitespaces are kept, default to false
+            if prevent_indent && with_leading_newline {
+                let t = template_stack.front_mut().unwrap();
+                // check the last element before current
+                if let Some(RawString(ref mut text)) = t.elements.last_mut() {
+                    // trim leading space for standalone statement
+                    *text = text
+                        .trim_end_matches(grammar::whitespace_matcher)
+                        .to_owned();
+                }
             }
+
+            // return true when the item is the first element in root template
+            current_span.start() == 0 || with_leading_newline
+        } else {
+            false
         }
     }
 
@@ -393,6 +436,7 @@ impl Template {
         source: &'a str,
         pair: Option<Pair<'a, Rule>>,
         trim_start: bool,
+        trim_start_line: bool,
     ) -> TemplateElement {
         let mut s = String::from(source);
 
@@ -422,42 +466,46 @@ impl Template {
 
         if trim_start {
             RawString(s.trim_start().to_owned())
+        } else if trim_start_line {
+            let s = s.trim_start_matches(grammar::whitespace_matcher);
+            RawString(grammar::strip_first_newline(s).to_owned())
         } else {
             RawString(s)
         }
     }
 
-    pub fn compile<'a>(source: &'a str) -> Result<Template, TemplateError> {
+    pub(crate) fn compile2<'a>(
+        source: &'a str,
+        options: TemplateOptions,
+    ) -> Result<Template, TemplateError> {
         let mut helper_stack: VecDeque<HelperTemplate> = VecDeque::new();
         let mut decorator_stack: VecDeque<DecoratorTemplate> = VecDeque::new();
         let mut template_stack: VecDeque<Template> = VecDeque::new();
 
         let mut omit_pro_ws = false;
+        // flag for newline removal of standalone statements
+        // this option is marked as true when standalone statement is detected
+        // then the leading whitespaces and newline of next rawstring will be trimed
+        let mut trim_line_required = false;
 
         let parser_queue = HandlebarsParser::parse(Rule::handlebars, source).map_err(|e| {
             let (line_no, col_no) = match e.line_col {
                 LineColLocation::Pos(line_col) => line_col,
                 LineColLocation::Span(line_col, _) => line_col,
             };
-            TemplateError::of(TemplateErrorReason::InvalidSyntax).at(source, line_no, col_no)
+            TemplateError::of(TemplateErrorReason::InvalidSyntax)
+                .at(source, line_no, col_no)
+                .in_template(options.name())
         })?;
 
-        // dbg!(parser_queue.clone());
+        // dbg!(parser_queue.clone().flatten());
 
         // remove escape from our pair queue
         let mut it = parser_queue
             .flatten()
             .filter(|p| {
                 // remove rules that should be silent but not for now due to pest limitation
-                !matches!(
-                    p.as_rule(),
-                    Rule::escape
-                        | Rule::brackets_open
-                        | Rule::brackets_close
-                        | Rule::double_brackets_open
-                        | Rule::double_brackets_close
-                        | Rule::comment_brackets_open
-                )
+                !matches!(p.as_rule(), Rule::escape)
             })
             .peekable();
         let mut end_pos: Option<Position<'_>> = None;
@@ -479,7 +527,12 @@ impl Template {
                     if rule == Rule::raw_block_end {
                         let mut t = Template::new();
                         t.push_element(
-                            Template::raw_string(&source[prev_end..span.start()], None, false),
+                            Template::raw_string(
+                                &source[prev_end..span.start()],
+                                None,
+                                false,
+                                trim_line_required,
+                            ),
                             line_no,
                             col_no,
                         );
@@ -487,11 +540,19 @@ impl Template {
                     } else {
                         let t = template_stack.front_mut().unwrap();
                         t.push_element(
-                            Template::raw_string(&source[prev_end..span.start()], None, false),
+                            Template::raw_string(
+                                &source[prev_end..span.start()],
+                                None,
+                                false,
+                                trim_line_required,
+                            ),
                             line_no,
                             col_no,
                         );
                     }
+
+                    // reset standalone statement marker
+                    trim_line_required = false;
                 }
 
                 let (line_no, col_no) = span.start_pos().line_col();
@@ -513,10 +574,14 @@ impl Template {
                                 &source[start..span.end()],
                                 Some(pair.clone()),
                                 omit_pro_ws,
+                                trim_line_required,
                             ),
                             line_no,
                             col_no,
                         );
+
+                        // reset standalone statement marker
+                        trim_line_required = false;
                     }
                     Rule::helper_block_start
                     | Rule::raw_block_start
@@ -554,6 +619,15 @@ impl Template {
                         }
                         omit_pro_ws = exp.omit_pro_ws;
 
+                        // standalone statement check, it also removes leading whitespaces of
+                        // previous rawstring when standalone statement detected
+                        trim_line_required = Template::process_standalone_statement(
+                            &mut template_stack,
+                            source,
+                            &span,
+                            true,
+                        );
+
                         let t = template_stack.front_mut().unwrap();
                         t.mapping.push(TemplateMapping(line_no, col_no));
                     }
@@ -567,6 +641,15 @@ impl Template {
                         }
                         omit_pro_ws = exp.omit_pro_ws;
 
+                        // standalone statement check, it also removes leading whitespaces of
+                        // previous rawstring when standalone statement detected
+                        trim_line_required = Template::process_standalone_statement(
+                            &mut template_stack,
+                            source,
+                            &span,
+                            true,
+                        );
+
                         let t = template_stack.pop_front().unwrap();
                         let h = helper_stack.front_mut().unwrap();
                         h.template = Some(t);
@@ -574,7 +657,12 @@ impl Template {
                     Rule::raw_block_text => {
                         let mut t = Template::new();
                         t.push_element(
-                            Template::raw_string(span.as_str(), Some(pair.clone()), omit_pro_ws),
+                            Template::raw_string(
+                                span.as_str(),
+                                Some(pair.clone()),
+                                omit_pro_ws,
+                                trim_line_required,
+                            ),
                             line_no,
                             col_no,
                         );
@@ -589,10 +677,10 @@ impl Template {
                     | Rule::decorator_block_end
                     | Rule::partial_block_end => {
                         let exp = Template::parse_expression(source, it.by_ref(), span.end())?;
+
                         if exp.omit_pre_ws {
                             Template::remove_previous_whitespace(&mut template_stack);
                         }
-
                         omit_pro_ws = exp.omit_pro_ws;
 
                         match rule {
@@ -615,6 +703,17 @@ impl Template {
                                 t.push_element(el, line_no, col_no);
                             }
                             Rule::decorator_expression | Rule::partial_expression => {
+                                // do not auto trim ident spaces for
+                                // partial_expression(>)
+                                let prevent_indent =
+                                    options.prevent_indent || rule != Rule::partial_expression;
+                                trim_line_required = Template::process_standalone_statement(
+                                    &mut template_stack,
+                                    source,
+                                    &span,
+                                    prevent_indent,
+                                );
+
                                 let decorator = DecoratorTemplate {
                                     name: exp.name,
                                     params: exp.params,
@@ -630,6 +729,15 @@ impl Template {
                                 t.push_element(el, line_no, col_no);
                             }
                             Rule::helper_block_end | Rule::raw_block_end => {
+                                // standalone statement check, it also removes leading whitespaces of
+                                // previous rawstring when standalone statement detected
+                                trim_line_required = Template::process_standalone_statement(
+                                    &mut template_stack,
+                                    source,
+                                    &span,
+                                    true,
+                                );
+
                                 let mut h = helper_stack.pop_front().unwrap();
                                 let close_tag_name = exp.name.as_name();
                                 if h.name.as_name() == close_tag_name {
@@ -648,10 +756,20 @@ impl Template {
                                             exp.name.debug_name(),
                                         ),
                                     )
-                                    .at(source, line_no, col_no));
+                                    .at(source, line_no, col_no)
+                                    .in_template(options.name()));
                                 }
                             }
                             Rule::decorator_block_end | Rule::partial_block_end => {
+                                // standalone statement check, it also removes leading whitespaces of
+                                // previous rawstring when standalone statement detected
+                                trim_line_required = Template::process_standalone_statement(
+                                    &mut template_stack,
+                                    source,
+                                    &span,
+                                    true,
+                                );
+
                                 let mut d = decorator_stack.pop_front().unwrap();
                                 let close_tag_name = exp.name.as_name();
                                 if d.name.as_name() == close_tag_name {
@@ -670,13 +788,21 @@ impl Template {
                                             exp.name.debug_name(),
                                         ),
                                     )
-                                    .at(source, line_no, col_no));
+                                    .at(source, line_no, col_no)
+                                    .in_template(options.name()));
                                 }
                             }
                             _ => unreachable!(),
                         }
                     }
                     Rule::hbs_comment_compact => {
+                        trim_line_required = Template::process_standalone_statement(
+                            &mut template_stack,
+                            source,
+                            &span,
+                            true,
+                        );
+
                         let text = span
                             .as_str()
                             .trim_start_matches("{{!")
@@ -685,6 +811,13 @@ impl Template {
                         t.push_element(Comment(text.to_owned()), line_no, col_no);
                     }
                     Rule::hbs_comment => {
+                        trim_line_required = Template::process_standalone_statement(
+                            &mut template_stack,
+                            source,
+                            &span,
+                            true,
+                        );
+
                         let text = span
                             .as_str()
                             .trim_start_matches("{{!--")
@@ -707,22 +840,32 @@ impl Template {
                     let t = template_stack.front_mut().unwrap();
                     t.push_element(RawString(text.to_owned()), line_no, col_no);
                 }
-                return Ok(template_stack.pop_front().unwrap());
+                let mut root_template = template_stack.pop_front().unwrap();
+                root_template.name = options.name;
+                return Ok(root_template);
             }
         }
     }
 
+    // These two compile functions are kept for compatibility with 4.x
+    // Template APIs in case that some developers are using them
+    // without registry.
+
+    pub fn compile(source: &str) -> Result<Template, TemplateError> {
+        Self::compile2(source, TemplateOptions::default())
+    }
+
     pub fn compile_with_name<S: AsRef<str>>(
         source: S,
         name: String,
     ) -> Result<Template, TemplateError> {
-        match Template::compile(source.as_ref()) {
-            Ok(mut t) => {
-                t.name = Some(name);
-                Ok(t)
-            }
-            Err(e) => Err(e.in_template(name)),
-        }
+        Self::compile2(
+            source.as_ref(),
+            TemplateOptions {
+                name: Some(name),
+                ..Default::default()
+            },
+        )
     }
 }
 
@@ -1050,7 +1193,7 @@ mod test {
         );
         let r = c.unwrap();
         // the \n after last raw block is dropped by pest
-        assert_eq!(r.elements.len(), 6);
+        assert_eq!(r.elements.len(), 9);
     }
 
     #[test]
